from django.http import HttpResponseBadRequest, HttpResponse
from hacore.models import Device, Protocol
from forms import DeviceForm, DeviceUpdateForm
from django.shortcuts import redirect, render_to_response, get_object_or_404
from django.core.urlresolvers import reverse
from django.views.decorators.csrf import csrf_exempt
import simplejson, logging
from django.db import IntegrityError
from django.contrib.auth.decorators import login_required
from auth import access_required
from django.http import QueryDict
from django.conf import settings

logger = logging.getLogger("rest")
logger.setLevel(settings.LOG_LEVEL)


#@user_passes_test(lambda u: u.is_superuser or u.groups.filter(name='Alumnos presenciales').exists())
def get(request, *args, **kwargs):

    if "did" in kwargs and "protocol" in kwargs:
        protocol = get_object_or_404(Protocol, name=kwargs["protocol"])
        obj = get_object_or_404(Device, protocol=protocol, did=kwargs["did"])

        response = HttpResponse(
            content=obj.to_json(),
            content_type="application/json",
        )

        response['Cache-Control'] = 'no-cache'
        return response

    if "protocol" in request.GET:
        if request.GET["protocol"] == "" or request.GET["protocol"] is None:
            return HttpResponseBadRequest("Invalid protocol")
        protocol = get_object_or_404(Protocol, name=request.GET["protocol"])

        data = [x.to_json() for x in protocol.devices.all()]

    elif "device_type" in request.GET:
        if request.GET["device_type"] == "" or request.GET["device_type"] is None:
            raise Exception("Invalid device_type")

        data = [x.to_json() for x in Device.objects.filter(device_type=request.GET["device_type"])]

    elif "status" in request.GET:
        if request.GET["status"] is None:
            raise Exception("Invalid status")

        data = [x.to_json() for x in Device.objects.filter(status=request.GET["status"])]

    else:
        raise Exception("Posible mandatory query filters 'did' or 'protocol' or 'device_type' or 'status'")

    response = render_to_response(
        "device/list.json",
        {"data": data},
        content_type="application/json",
    )
    response['Cache-Control'] = 'no-cache'
    return response


def put(request, protocol, did):

    protocol = get_object_or_404(Protocol, name=protocol)
    obj = get_object_or_404(Device, protocol=protocol, did=did)

    qd = QueryDict(request.body, request.encoding)
    form = DeviceUpdateForm(qd, instance=obj)

    if form.is_valid():
        obj = form.save(commit=False)

        obj.save()

        response = redirect(reverse('device_by_id', kwargs={"protocol": obj.protocol, "did": obj.id}))
        response.content_type = "application/json"
        return response
    logger.debug("PUT view: errores en el form %s" % form.errors)
    return HttpResponseBadRequest(
        simplejson.dumps({"errors": [x for x in form.errors]}),
        content_type="application/json")


def delete(request, protocol, did):
    protocol = get_object_or_404(Protocol, name=protocol)
    get_object_or_404(Device, protocol=protocol, did=did).delete()
    return HttpResponse(status=204)


def post(request):

    if "protocol" not in request.POST:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["protocol"]}),
            content_type="application/json",
            )
    protocol = get_object_or_404(Protocol, name=request.POST["protocol"])

    form = DeviceForm(request.POST)
    if form.is_valid():
        exec "from %s import validate_address" % protocol.validate_address_module
        obj = form.save(commit=False)
        obj.protocol = protocol
        try:
            validate_address(obj.did)
            obj.save()
        except ValueError:
            return HttpResponseBadRequest(
                content=simplejson.dumps({"errors": ["did"]}),
                content_type="application/json",
                )
        except IntegrityError:
            return HttpResponse(
                status=409,
                content=simplejson.dumps({"conflicting": ["did", "protocol"]}),
                content_type="application/json",
                )
        response = redirect(reverse('device_by_id', kwargs={"protocol": obj.protocol, "did": obj.id}))
        response.content_type = "application/json"
        return response
    return HttpResponseBadRequest(
        content=simplejson.dumps({"errors": [x for x in form.errors]}),
        content_type="application/json",)


@csrf_exempt
@access_required
def entrance(request, *args, **kwargs):

    if request.method == "GET":
        return get(request, *args, **kwargs)

    if request.method == "PUT":
        try:
            did = kwargs["did"]
            protocol = kwargs["protocol"]
            return put(request, protocol, did)
        except KeyError:
            return HttpResponseBadRequest(simplejson.dumps({"errors": ["Device id(did) or protocol missing"]}))

        return put(request)

    if request.method == "DELETE":
        try:
            did = kwargs["did"]
            protocol = kwargs["protocol"]
            return delete(request, protocol, did)
        except KeyError:
            return HttpResponseBadRequest(simplejson.dumps({"errors": ["Device id(did) or protocol missing"]}))

    if request.method == "POST":
        return post(request)



from django.conf.urls import patterns, include, url


urlpatterns = patterns('harest',
    url(r'^manage/device/(?P<protocol>[a-z0-9A-Z]{3,4})/(?P<did>[\d\w]+)', 'views_device.entrance', name='device_by_id'),
    url(r'^manage/device/?', 'views_device.entrance', name='device'),

   # url(r'^test/?', 'views_cmds.test', name='test'),

    url(r'^manage/protocol/?', 'views_protocol.entrance', name='protocol'),

    url(r'^cmd/pl_all_lights_off/(?P<protocol>[a-z0-9A-Z]{3,4})/(?P<group>[\d\w]+)/?', 'views_cmds.pl_all_lights_off', name="pl_all_lights_off"),
    url(r'^cmd/pl_all_lights_on/(?P<protocol>[a-z0-9A-Z]{3,4})/(?P<group>[\d\w]+)/?', 'views_cmds.pl_all_lights_on', name="pl_all_lights_on"),
    url(r'^cmd/pl_switch/(?P<protocol>[a-z0-9A-Z]{3,4})/(?P<did>[\d\w]+)/?', 'views_cmds.pl_switch', name="pl_switch"),
    url(r'^cmd/pl_dim/(?P<protocol>[a-z0-9A-Z]{3,4})/(?P<did>[\d\w]+)/?', 'views_cmds.pl_dim', name="pl_dim"),
    url(r'^cmd/pl_bri/(?P<protocol>[a-z0-9A-Z]{3,4})/(?P<did>[\d\w]+)/?', 'views_cmds.pl_bri', name="pl_bri"),
)



from hacore.models import Protocol, Device
from django.shortcuts import get_object_or_404, redirect
from django.http import HttpResponseBadRequest, HttpResponse
from django.http import QueryDict
from django.views.decorators.csrf import csrf_exempt
from auth import access_required
from django.core.urlresolvers import reverse
import simplejson


@csrf_exempt
@access_required
def pl_all_lights_on(request, protocol, group):
    if request.method != "PUT":
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Only PUT HTTP verb accepted for pl_all_lights_on command!!, arrived: %s" % request.method, ]}),
            content_type="application/json",)
    try:
        protocol = Protocol.objects.get(name=protocol)
    except Protocol.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Protocol %s not found" % protocol, ]}),
            content_type="application/json",)
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching protocol %s: %s" % (protocol, er), ]}),
            content_type="application/json",)

    exec "from %s.cmds import pl_all_lights_on" % protocol.module

    try:
        ret = pl_all_lights_on(group)
    except ValueError as ex:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": [str(ex), ]}),
            content_type="application/json",)

    for device in Device.objects.filter(did__istartswith=group):
        device.status = 100
        device.save()
    response = HttpResponse(simplejson.dumps({"status": "ok"}))
    response.content_type = "application/json"
    return response


@csrf_exempt
@access_required
def pl_all_lights_off(request, protocol, group):
    if request.method != "PUT":
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Only PUT HTTP verb accepted for pl_all_lights_off command!!, arrived: %s" % request.method, ]}),
            content_type="application/json",)
    try:
        protocol = Protocol.objects.get(name=protocol)
    except Protocol.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Protocol %s not found" % protocol, ]}),
            content_type="application/json",)
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching protocol %s: %s" % (protocol, er), ]}),
            content_type="application/json",)

    exec "from %s.cmds import pl_all_lights_off" % protocol.module

    try:
        ret = pl_all_lights_off(group)
    except ValueError as ex:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": [str(ex), ]}),
            content_type="application/json",)

    for device in Device.objects.filter(did__istartswith=group):
        device.status = 0
        device.save()
    response = HttpResponse(simplejson.dumps({"status": "ok"}))
    response.content_type = "application/json"
    return response


@csrf_exempt
@access_required
def pl_switch(request, protocol, did):

    if request.method != "PUT":
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Only PUT HTTP verb accepted for pl_switch command!!, arrived: %s" % request.method, ]}),
            content_type="application/json",)

    qd = QueryDict(request.body, request.encoding)
    if "value" not in qd:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Switch command needs a value to be set", ]}),
            content_type="application/json",)
    value = qd["value"].lower()

    try:
        protocol = Protocol.objects.get(name=protocol)
    except Protocol.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Protocol %s not found" % protocol, ]}),
            content_type="application/json",)
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching protocol %s: %s" % (protocol, er), ]}),
            content_type="application/json",)

    try:
        device = Device.objects.get(did=did)
    except Device.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Device (did=%s) not found" % did, ]}),
            content_type="application/json",)
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching device (did=%s): %s" % (did, er), ]}),
            content_type="application/json",
            )


    exec "from %s.cmds import pl_switch" % protocol.module

    try:
        ret = pl_switch(device.did, value)
    except ValueError as ex:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": [str(ex), ]}),
            content_type="application/json",
            )

    device.status = 0 if value == "off" else 100
    device.save()
    response = redirect(reverse('device_by_id', kwargs={"protocol": device.protocol, "did": device.did}))
    response.content_type = "application/json"
    return response


@csrf_exempt
@access_required
def pl_dim(request, protocol, did):

    if request.method != "POST":
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Only POST(no idempotent) HTTP verb accepted for pl_dim command!!, arrived: %s" % request.method, ]}),
            content_type="application/json",
            )

    if "value" not in request.POST:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Dim command needs a value to be set", ]}),
            content_type="application/json", )
    value = request.POST["value"]

    try:
        protocol = Protocol.objects.get(name=protocol)
    except Protocol.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Protocol %s not found" % protocol, ]}),
            content_type="application/json",
            )
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching protocol %s: %s" % (protocol, er), ]}),
            content_type="application/json",
            )

    try:
        device = Device.objects.get(did=did, device_type="dimmer")
    except Device.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Device (did=%s, type=dimmer) not found" % did, ]}),
            content_type="application/json",
            )
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching device (did=%s, type=dimmer): %s" % (did, er), ]}),
            content_type="application/json",
            )

    exec "from %s.cmds import pl_dim" % protocol.module

    try:
        ret = pl_switch(device.did, value)
    except ValueError as ex:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": [str(ex), ]}),
            content_type="application/json",
            )
    device.status = 0 if value == "off" else 100
    device.save()
    response = redirect(reverse('device_by_id', kwargs={"protocol": device.protocol, "did": device.did}))
    response.content_type = "application/json"
    return response


@csrf_exempt
@access_required
def pl_bri(request, protocol, did):

    if request.method != "POST":
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Only POST(no idempotent) HTTP verb accepted for pl_bri command!!, arrived: %s" % request.method, ]}),
            content_type="application/json",
            )
    if "value" not in request.POST:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Bri command needs a value to be set", ]}),
            content_type="application/json",
            )
    value = request.POST["value"]

    try:
        protocol = Protocol.objects.get(name=protocol)
    except Protocol.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Protocol %s not found" % protocol, ]}),
            content_type="application/json",)
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching protocol %s: %s" % (protocol, er), ]}),
            content_type="application/json",)

    try:
        device = Device.objects.get(did=did, device_type="dimmer")
    except Device.DoesNotExist:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Device (did=%s, type=dimmer) not found" % did, ]}),
            content_type="application/json",
            )
    except Exception as er:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error while fetching device (did=%s, type=dimmer): %s" % (did, er), ]}),
            content_type="application/json",)


    exec "from %s.cmds import pl_bri" % protocol.module

    try:
        ret = pl_bri(device.did, value)
    except ValueError as ex:
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": [str(ex), ]}),
            content_type="application/json",)

    device.status = 0 if value == "off" else 100
    device.save()
    response = redirect(reverse('device_by_id', kwargs={"protocol": device.protocol, "did": device.did}))
    response.content_type = "application/json"
    return response



from django.db import models
import simplejson

DEVICE_TYPES = ("switch", "dimmer")


class Protocol(models.Model):

    name = models.CharField(max_length=10, unique=True)
    gobj_name = models.CharField(max_length=10, unique=True)
    module = models.CharField(max_length=30, unique=True)
    validate_address_module = models.CharField(max_length=30)

    def __unicode__(self):
        return u"%s" % self.name

    def to_json(self):
        return simplejson.dumps({"name": self.name})


class Device(models.Model):
    caption = models.CharField(max_length=100)
    #device id
    did = models.CharField(max_length=100)
    status = models.IntegerField(blank=True, null=True)
    device_type = models.CharField(max_length=10, choices=(('switch', 'switch'), ('dimmer', 'dimmer'), ('sensor', 'sensor')))
    protocol = models.ForeignKey(Protocol, related_name="devices")

    def to_json(self):
        return simplejson.dumps({
            "caption": self.caption,
            "did": self.did,
            "status": self.status,
            "device_type": self.device_type,
            "protocol": self.protocol.__unicode__(),
            })

    def __unicode__(self):
        return "%s" % self.caption

    def cmd(self, cmd):
        pass

    class Meta:
        unique_together = ("protocol", "did")





from django.contrib.auth import authenticate, login
from django.http import HttpResponse, HttpResponseForbidden
from functools import wraps
import simplejson


def access_required(func, *args, **kwargs):

    def decorator(*args, **kwargs):

        if "HTTP_USERNAME" not in args[0].META or "HTTP_PASSWORD" not in args[0].META:
            return HttpResponseForbidden(
                simplejson.dumps({"errors": ["HTTP_USERNAME or HTTP_PASSWORD headers not sent", ]}),
                content_type="application/json",)

        username = args[0].META["HTTP_USERNAME"]
        password = args[0].META["HTTP_PASSWORD"]
        user = authenticate(username=username, password=password)

        if user is not None:
            if user.is_active:
                return func(*args, **kwargs)
        return HttpResponseForbidden(
            simplejson.dumps({"errors": ["username/password not valid", ]}),
            content_type="application/json",)

    return decorator



from django.forms import ModelForm
from hacore.models import Device
from django.core.exceptions import ValidationError
from django.forms.fields import FileField


class DeviceForm(ModelForm):
    class Meta:
        model = Device
        exclude = ["protocol"]


class DeviceUpdateForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(DeviceUpdateForm, self).__init__(*args, **kwargs)
        self.fields["device_type"].required = False
        self.fields["caption"].required = False

    class Meta:
        model = Device
        exclude = ["did", "status", "protocol"]



from django.contrib import admin
from models import *


class ProtocolAdmin(admin.ModelAdmin):
    pass
admin.site.register(Protocol, ProtocolAdmin)


class DeviceAdmin(admin.ModelAdmin):
    pass
admin.site.register(Device, DeviceAdmin)



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import LiveServerTestCase
from selenium.webdriver.firefox.webdriver import WebDriver
#from django.test import TestCase
from django.core.urlresolvers import reverse
import requests
import os
from hacore.models import Protocol
from django.conf import settings


def show_response(response):
    print "Return code: %s\nContent: %s\n" % (response.status_code, response.content)


def populate_initial_data():
    Protocol(name="X10", module="hautomation_x10.cmds", validate_address_module="hautomation_x10.utils").save()


def new_device(protocol, did, device_type, url):
    data = {
            "caption": "created automatic",
            "did": did,
            "device_type": device_type,
            "protocol": protocol,
        }
    return requests.post("%s%s" % (url, reverse("device")), data=data)


def del_device(did, url):
    return requests.delete("%s%s" % (url, reverse("device_by_id", kwargs={"did": did})))


class RestTest(LiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        cls.selenium = WebDriver()
        super(RestTest, cls).setUpClass()

        #Protocol(name="KXN", ").save()

        try:
            if settings.TEST_SERVER is not None and settings.TEST_SERVER != "":
                cls.live_server_url = settings.TEST_SERVER
            else:
                populate_initial_data()
        except AttributeError:
            populate_initial_data()


    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(RestTest, cls).tearDownClass()


class DeviceTest(RestTest):

    def test_post(self):
        print "Running DeviceTest.post(create)....................................."
        data = {
            "caption": "Persiana terraza",
            "did": "A1",
            "device_type": "dimmer",
            "protocol": "saa10"
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 404, "Not validating post request properly(sent wrong protocol), should returned 400, instead: %s" % r.status_code)

        data = {
            "caption": "Persiana terraza",
            "did": "A1",
            "protocol": "X10",
            "device_type": "claxon",
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 400, "Not validating post request properly(sent wrong device_type), should returned 400, instead: %s" % r.status_code)

        data = {
            "caption": "Persiana terraza",
            "protocol": "X10",
            "device_type": "dimmer",
            "did": "sa55",
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 400, "Not validating post request properly(sent wrong did), should returned 400, instead: %s" % r.status_code)

        data = {
            "did": "z5",
            "caption": "Persiana terraza",
            "protocol": "X10",
            "device_type": "dimmer",
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 400, "Not validating post request properly(sent wrong did), should returned 400, instead: %s" % r.status_code)

        data = {
            "did": "5a",
            "caption": "Persiana terraza",
            "protocol": "X10",
            "device_type": "dimmer",
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 400, "Not validating post request properly(sent wrong did), should returned 400, instead: %s" % r.status_code)

        data = {
            "did": "zz",
            "caption": "Persiana terraza",
            "protocol": "X10",
            "device_type": "dimmer",
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 400, "Not validating post request properly(sent wrong did), should returned 400, instead: %s" % r.status_code)

        data = {
            "caption": "Persiana terraza",
            "did": "A1",
            "device_type": "dimmer",
            "protocol": "X10"
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 200, "Not creating properly devices, returned: %s" % r.status_code)

        data = {
            "caption": "Persiana terraza",
            "did": "A1",
            "device_type": "dimmer",
            "protocol": "X10"
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)
        self.assertTrue(r.status_code == 409, "Not creating properly devices, returned: %s(should return 409(conflict))" % r.status_code)

        del_device("A1", self.live_server_url)

    def test_delete(self):
        print "running DeviceTest.delete ........................................"
        r = new_device("X10", "A3", "switch", self.live_server_url)

        print "errrr r:%s" % r.json()
        id = r.json()["did"]

        r = requests.delete("%s%s" % (self.live_server_url, reverse("device_by_id", args=[id])))
        self.assertTrue(r.status_code == 204, "No deleting property device, status returned: %s(should return 204)" % r.status_code)

        r = requests.delete("%s%s" % (self.live_server_url, reverse("device_by_id", args=[id])))
        self.assertTrue(r.status_code == 404, "No deleting property device, status returned: %s " % r.status_code)

    def test_get(self):
        print "Running DeviceTest.get.................................................."
        data = {
            "caption": "Persiana terraza",
            "did": "A2",
            "device_type": "dimmer",
            "protocol": "X10"
        }
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)

        data["did"] = "A21"
        data["caption"] = "persiana salon"
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)

        data["did"] = "A3"
        data["caption"] = "persiana cuarto"
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)

        data["did"] = "A4"
        data["caption"] = "persiana pasillo"
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)

        r = requests.get("%s%s?device_type=dimmer" % (self.live_server_url, reverse("device")))
        print r.json()
        self.assertTrue(len(r.json()) == 4, "Not creating devices properly or not retrieving")

        r = requests.get("%s%s?device_type=switch" % (self.live_server_url, reverse("device")))

        self.assertTrue(len(r.json()) == 0, "Not retrieving devices properly, asked for switch devices(only dimmer at db), should return 0, returned: %s" % len(r.json()))

        data["did"] = "A5"
        data["caption"] = "luz pasillo"
        data["device_type"] = "switch"
        r = requests.post("%s%s" % (self.live_server_url, reverse("device")), data=data)

        r = requests.get("%s%s?device_type=switch" % (self.live_server_url, reverse("device")))

        self.assertTrue(len(r.json()) == 1, "Not retrieving switches properly, should return 1, returned: %s" % len(r.json()))

        del_device("A2", self.live_server_url)
        del_device("A21", self.live_server_url)
        del_device("A3", self.live_server_url)
        del_device("A4", self.live_server_url)


class CmdTest(RestTest):
    def test_pl_switch(self):
        print "Running CmdTests.pl_switch ..............................................."
        r = new_device("X10", "A1", "switch", self.live_server_url)


        url = {
            "host": self.live_server_url,
            "path": reverse("pl_switch", kwargs={
                    "protocol": "X10",
                    "did": "A1",
                }),

        }
        data = {"value": "ON"}
        #r = requests.put("{host}{path}".format(**url), data=data)
        r = requests.request("PUT", "{host}{path}".format(**url), data=data)

        self.assertTrue(r.status_code == 200, "Not properly validating pl_switch values, sent uppercaer ON and didn't accept it\n%s" % show_response(r))

        data = {"value": "on"}
        r = requests.request("PUT", "{host}{path}".format(**url), data=data)
        self.assertTrue(r.status_code == 200, "Not properly validating pl_switch values, sent on and didn't accept it\n%s" % show_response(r))

        data = {"value": "OFF"}
        r = requests.request("PUT", "{host}{path}".format(**url), data=data)
        self.assertTrue(r.status_code == 200, "Not properly validating pl_switch values, sent uppercaer OFF and didn't accept it\n%s" % show_response(r))

        data = {"value": "off"}
        r = requests.request("PUT", "{host}{path}".format(**url), data=data)
        self.assertTrue(r.status_code == 200, "Not properly validating pl_switch values, sent on and didn't accept it\n%s" % show_response(r))

        data = {"value": "Odgdfg"}
        r = requests.request("PUT", "{host}{path}".format(**url), data=data)
        self.assertTrue(r.status_code == 400, "Not properly validating pl_switch values, sent invalid value and it accepted it\n%s" % show_response(r))


        url = {
            "host": self.live_server_url,
            "path": reverse("pl_switch", kwargs={
                    "protocol": "X10",
                    "did": "A1hh",
                }),

        }
        data = {"value": "Odgdfg"}
        r = requests.request("PUT", "{host}{path}".format(**url), data=data)
        self.assertTrue(r.status_code == 400, "Not properly validating X10 address, sent invalid did and it accepted it\n%s" % show_response(r))


        del_device("A1", self.live_server_url)



from django.http import Http404, HttpResponseNotFound, HttpResponseServerError
import simplejson, logging, traceback

from django.conf import settings

logger = logging.getLogger("rest")
logger.setLevel(settings.LOG_LEVEL)


class Exception2Json():
    """
    Middleware meant to process exception found at views
    """
    def process_exception(self, request, exception):
        logger.error("EXCEPTION: %s" % exception)
        if isinstance(exception, Http404):
            return HttpResponseNotFound(
                content=simplejson.dumps({"errors": [str(exception)]}),
                content_type="application/json",)

        logger.error(traceback.format_exc())
        msg = "Server error: %s" % exception if settings.DEBUG else "Server error"
        return HttpResponseServerError(
            content=simplejson.dumps({"errors": [msg, ]}),
            content_type="application/json",)



from hacore.models import Protocol
from django.shortcuts import render_to_response
from auth import access_required
from django.http import HttpResponseBadRequest
import simplejson
import logging
from django.conf import settings


logger = logging.getLogger("rest")
logger.setLevel(settings.LOG_LEVEL)


def get(request, *args, **kwargs):


    data = [x.to_json() for x in Protocol.objects.all()]

    response = render_to_response(
        "device/list.json",
        {"data": data},
        content_type="application/json",
    )

    response['Cache-Control'] = 'no-cache'
    return response



@access_required
def entrance(request, *args, **kwargs):

    try:
        if request.method == "GET":
            return get(request, *args, **kwargs)

        return HttpResponseBadRequest(simplejson.dumps({"errors": ["Only GET HTTP verb accepted for Protocol resources"]}))
    except Exception as ex:
        logger.error(ex)
        raise ex